<?php

namespace Drupal\paragraphs_blokkli_graphql\Plugin\GraphQL\DataProducer;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Procudes the list of possible paragraph conversion.
 *
 * @DataProducer(
 *   id = "paragraphs_blokkli_field_config",
 *   name = @Translation("Paragraphs Blokkli Field Config"),
 *   description = @Translation("Procudes a list of all existing paragraph fields."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("The list of paragraph fields.")
 *   ),
 * )
 */
class ParagraphsBlokkliFieldConfig extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_field.manager')
    );
  }

  /**
   * EntityLoad constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param array $pluginDefinition
   *   The plugin definition array.
   * @param EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    array $pluginDefinition,
    protected EntityFieldManagerInterface $entityFieldManager
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
  }

  /**
   * Resolver.
   */
  public function resolve() {
    $fieldConfigs = [];
    $fieldMap = $this->entityFieldManager->getFieldMapByFieldType('entity_reference_revisions');
    foreach ($fieldMap as $entityType => $fields) {
      foreach ($fields as $fieldName => $fieldOptions) {
        foreach ($fieldOptions['bundles'] as $bundle) {
          $fields = $this->entityFieldManager->getFieldDefinitions($entityType, $bundle);
          /** @var \Drupal\Core\Field\BaseFieldDefinition|null $field */
          $field = $fields[$fieldName] ?? NULL;
          if ($field && $field->getType() === 'entity_reference_revisions') {
            $settings = $field->getSettings();
            $targetType = $settings['target_type'] ?? NULL;
            $storageDefinition = $field->getFieldStorageDefinition();

            if ($targetType === 'paragraph') {
              $targetBundles = $settings['handler_settings']['target_bundles'] ?? [];
              $fieldConfigs[] = [
                'name' => $fieldName,
                'entityType' => $entityType,
                'entityBundle' => $bundle,
                'label' => $field instanceof FieldConfig ? $field->label() : $storageDefinition->getLabel(),
                'cardinality' => $storageDefinition->getCardinality(),
                'allowedBundles' => $targetBundles,
                'canEdit' => TRUE,
              ];
            }
          }
        }
      }
    }

    return $fieldConfigs;
  }

}
