<?php

namespace Drupal\paragraphs_blokkli_graphql\Plugin\GraphQL\DataProducer;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\paragraphs_blokkli\EditStateInterface;
use Drupal\paragraphs_blokkli\ParagraphMutationPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Produce a new paragraphs blokkli mutation.
 *
 * @DataProducer(
 *   id = "paragraphs_blokkli_mutation",
 *   name = @Translation("Paragraphs Blokkli Mutation"),
 *   description = @Translation("Creates a new paragraphs blokkli mutation."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("The mutation.")
 *   ),
 *   consumes = {
 *     "state" = @ContextDefinition("any",
 *       label = @Translation("The paragraphs edit state.")
 *     ),
 *     "pluginId" = @ContextDefinition("string",
 *       label = @Translation("The mutation plugin id.")
 *     ),
 *     "values" = @ContextDefinition("any",
 *       label = @Translation("The values for the mutation."),
 *     ),
 *   }
 * )
 */
class ParagraphsBlokkliMutation extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The paragraph mutation plugin manager.
   */
  protected ParagraphMutationPluginManager $mutationPluginManager;

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.paragraph_mutation'),
    );
  }

  /**
   * EntityLoad constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param array $pluginDefinition
   *   The plugin definition array.
   * @param ParagraphMutationPluginManager $mutationPluginManager
   *   The paragraph mutation plugin manager.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    array $pluginDefinition,
    ParagraphMutationPluginManager $mutationPluginManager
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->mutationPluginManager = $mutationPluginManager;
  }

  /**
   * Resolver.
   */
  public function resolve(EditStateInterface $state, string $pluginId, array $values) {
    /** @var \Drupal\Core\Render\RendererInterface $renderer **/
    $renderer = \Drupal::service('renderer');
    $context = new RenderContext();
    return $renderer->executeInRenderContext($context, function () use ($state, $pluginId, $values) {
      $mutation = $this->mutationPluginManager->createInstance($pluginId, $values);
      $state->addMutation($mutation);
      $state->getMutatedState();
      return $state;
    });
  }

}
