<?php

namespace Drupal\paragraphs_blokkli;

class ParagraphsBlokkliMutatedState {

  /**
   * Constructs a new ParagraphsBlokkliMutatedState.
   *
   * @param ParagraphsBlokkliMutatedField[] $fields
   *   The mutated fields.
   * @param array $behaviorSettings
   *   The mutated behavior settings, keyed by paragraph UUID.
   * @param \Symfony\Component\Validator\ConstraintViolationInterface[] $violations
   *   The violations.
   */
  public function __construct(
    protected array $fields,
    protected array $behaviorSettings,
    protected array $violations
  ) {
  }

  /**
   * Get the mutated field item lists.
   *
   * @return ParagraphsBlokkliMutatedField[]
   *   The mutated field item lists.
   */
  public function getFields(): array {
    return $this->fields;
  }

  /**
   * Get the behavior settings.
   *
   * @return array
   *   The behavior settings keyed by paragraph UUID.
   */
  public function getBehaviorSettings(): array {
    return $this->behaviorSettings;
  }

  /**
   * Get the violations.
   *
   * @return \Symfony\Component\Validator\ConstraintViolationInterface[]
   *   The violations.
   */
  public function getViolations(): array {
    return $this->violations;
  }

}
